#line 1 "ExprSpecParserLexIn.cpp"

#line 3 "ExprSpecParserLexIn.cpp"

#define  ExprSpecYY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define ExprSpecYY_FLEX_MAJOR_VERSION 2
#define ExprSpecYY_FLEX_MINOR_VERSION 6
#define ExprSpecYY_FLEX_SUBMINOR_VERSION 4
#if ExprSpecYY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* begin standard C++ headers. */

/* TODO: this is always defined, so inline it */
#define ExprSpecconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define ExprSpecnoreturn __attribute__((__noreturn__))
#else
#define ExprSpecnoreturn
#endif

/* Returned upon end-of-file. */
#define ExprSpecYY_NULL 0

/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define ExprSpecYY_SC_TO_UI(c) ((ExprSpecYY_CHAR) (c))

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (ExprSpec_start) = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The ExprSpecYYSTATE alias is for lex
 * compatibility.
 */
#define ExprSpecYY_START (((ExprSpec_start) - 1) / 2)
#define ExprSpecYYSTATE ExprSpecYY_START
/* Action number for EOF rule of a given start state. */
#define ExprSpecYY_STATE_EOF(state) (ExprSpecYY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define ExprSpecYY_NEW_FILE ExprSpecrestart( ExprSpecin  )
#define ExprSpecYY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef ExprSpecYY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, ExprSpecYY_BUF_SIZE is 2*ExprSpecYY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define ExprSpecYY_BUF_SIZE 32768
#else
#define ExprSpecYY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define ExprSpecYY_STATE_BUF_SIZE   ((ExprSpecYY_BUF_SIZE + 2) * sizeof(ExprSpec_state_type))

#ifndef ExprSpecYY_TYPEDEF_ExprSpecYY_BUFFER_STATE
#define ExprSpecYY_TYPEDEF_ExprSpecYY_BUFFER_STATE
typedef struct ExprSpec_buffer_state *ExprSpecYY_BUFFER_STATE;
#endif

#ifndef ExprSpecYY_TYPEDEF_ExprSpecYY_SIZE_T
#define ExprSpecYY_TYPEDEF_ExprSpecYY_SIZE_T
typedef size_t ExprSpec_size_t;
#endif

extern int ExprSpecleng;

extern FILE *ExprSpecin, *ExprSpecout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define ExprSpecYY_LESS_LINENO(n)
    #define ExprSpecYY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define ExprSpecless(n) \
	do \
		{ \
		/* Undo effects of setting up ExprSpectext. */ \
        int ExprSpecless_macro_arg = (n); \
        ExprSpecYY_LESS_LINENO(ExprSpecless_macro_arg);\
		*ExprSpec_cp = (ExprSpec_hold_char); \
		ExprSpecYY_RESTORE_ExprSpecYY_MORE_OFFSET \
		(ExprSpec_c_buf_p) = ExprSpec_cp = ExprSpec_bp + ExprSpecless_macro_arg - ExprSpecYY_MORE_ADJ; \
		ExprSpecYY_DO_BEFORE_ACTION; /* set up ExprSpectext again */ \
		} \
	while ( 0 )
#define unput(c) ExprSpecunput( c, (ExprSpectext_ptr)  )

#ifndef ExprSpecYY_STRUCT_ExprSpecYY_BUFFER_STATE
#define ExprSpecYY_STRUCT_ExprSpecYY_BUFFER_STATE
struct ExprSpec_buffer_state
	{
	FILE *ExprSpec_input_file;

	char *ExprSpec_ch_buf;		/* input buffer */
	char *ExprSpec_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int ExprSpec_buf_size;

	/* Number of characters read into ExprSpec_ch_buf, not including EOB
	 * characters.
	 */
	int ExprSpec_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int ExprSpec_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int ExprSpec_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int ExprSpec_at_bol;

    int ExprSpec_bs_lineno; /**< The line count. */
    int ExprSpec_bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int ExprSpec_fill_buffer;

	int ExprSpec_buffer_status;

#define ExprSpecYY_BUFFER_NEW 0
#define ExprSpecYY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as ExprSpecYY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via ExprSpecrestart()), so that the user can continue scanning by
	 * just pointing ExprSpecin at a new input file.
	 */
#define ExprSpecYY_BUFFER_EOF_PENDING 2

	};
#endif /* !ExprSpecYY_STRUCT_ExprSpecYY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t ExprSpec_buffer_stack_top = 0; /**< index of top of stack. */
static size_t ExprSpec_buffer_stack_max = 0; /**< capacity of stack. */
static ExprSpecYY_BUFFER_STATE * ExprSpec_buffer_stack = NULL; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define ExprSpecYY_CURRENT_BUFFER ( (ExprSpec_buffer_stack) \
                          ? (ExprSpec_buffer_stack)[(ExprSpec_buffer_stack_top)] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define ExprSpecYY_CURRENT_BUFFER_LVALUE (ExprSpec_buffer_stack)[(ExprSpec_buffer_stack_top)]

/* ExprSpec_hold_char holds the character lost when ExprSpectext is formed. */
static char ExprSpec_hold_char;
static int ExprSpec_n_chars;		/* number of characters read into ExprSpec_ch_buf */
int ExprSpecleng;

/* Points to current character in buffer. */
static char *ExprSpec_c_buf_p = NULL;
static int ExprSpec_init = 0;		/* whether we need to initialize */
static int ExprSpec_start = 0;	/* start state number */

/* Flag which is used to allow ExprSpecwrap()'s to do buffer switches
 * instead of setting up a fresh ExprSpecin.  A bit of a hack ...
 */
static int ExprSpec_did_buffer_switch_on_eof;

void ExprSpecrestart ( FILE *input_file  );
void ExprSpec_switch_to_buffer ( ExprSpecYY_BUFFER_STATE new_buffer  );
ExprSpecYY_BUFFER_STATE ExprSpec_create_buffer ( FILE *file, int size  );
void ExprSpec_delete_buffer ( ExprSpecYY_BUFFER_STATE b  );
void ExprSpec_flush_buffer ( ExprSpecYY_BUFFER_STATE b  );
void ExprSpecpush_buffer_state ( ExprSpecYY_BUFFER_STATE new_buffer  );
void ExprSpecpop_buffer_state ( void );

static void ExprSpecensure_buffer_stack ( void );
static void ExprSpec_load_buffer_state ( void );
static void ExprSpec_init_buffer ( ExprSpecYY_BUFFER_STATE b, FILE *file  );
#define ExprSpecYY_FLUSH_BUFFER ExprSpec_flush_buffer( ExprSpecYY_CURRENT_BUFFER )

ExprSpecYY_BUFFER_STATE ExprSpec_scan_buffer ( char *base, ExprSpec_size_t size  );
ExprSpecYY_BUFFER_STATE ExprSpec_scan_string ( const char *ExprSpec_str  );
ExprSpecYY_BUFFER_STATE ExprSpec_scan_bytes ( const char *bytes, int len  );

void *ExprSpecalloc ( ExprSpec_size_t  );
void *ExprSpecrealloc ( void *, ExprSpec_size_t  );
void ExprSpecfree ( void *  );

#define ExprSpec_new_buffer ExprSpec_create_buffer
#define ExprSpec_set_interactive(is_interactive) \
	{ \
	if ( ! ExprSpecYY_CURRENT_BUFFER ){ \
        ExprSpecensure_buffer_stack (); \
		ExprSpecYY_CURRENT_BUFFER_LVALUE =    \
            ExprSpec_create_buffer( ExprSpecin, ExprSpecYY_BUF_SIZE ); \
	} \
	ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_is_interactive = is_interactive; \
	}
#define ExprSpec_set_bol(at_bol) \
	{ \
	if ( ! ExprSpecYY_CURRENT_BUFFER ){\
        ExprSpecensure_buffer_stack (); \
		ExprSpecYY_CURRENT_BUFFER_LVALUE =    \
            ExprSpec_create_buffer( ExprSpecin, ExprSpecYY_BUF_SIZE ); \
	} \
	ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_at_bol = at_bol; \
	}
#define ExprSpecYY_AT_BOL() (ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_at_bol)

/* Begin user sect3 */

#define ExprSpecwrap() (/*CONSTCOND*/1)
#define ExprSpecYY_SKIP_ExprSpecYYWRAP
typedef flex_uint8_t ExprSpecYY_CHAR;

FILE *ExprSpecin = NULL, *ExprSpecout = NULL;

typedef int ExprSpec_state_type;

extern int ExprSpeclineno;
int ExprSpeclineno = 1;

extern char *ExprSpectext;
#ifdef ExprSpectext_ptr
#undef ExprSpectext_ptr
#endif
#define ExprSpectext_ptr ExprSpectext

static ExprSpec_state_type ExprSpec_get_previous_state ( void );
static ExprSpec_state_type ExprSpec_try_NUL_trans ( ExprSpec_state_type current_state  );
static int ExprSpec_get_next_buffer ( void );
static void ExprSpecnoreturn ExprSpec_fatal_error ( const char* msg  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up ExprSpectext.
 */
#define ExprSpecYY_DO_BEFORE_ACTION \
	(ExprSpectext_ptr) = ExprSpec_bp; \
	ExprSpecleng = (int) (ExprSpec_cp - ExprSpec_bp); \
	(ExprSpec_hold_char) = *ExprSpec_cp; \
	*ExprSpec_cp = '\0'; \
	(ExprSpec_c_buf_p) = ExprSpec_cp;
#define ExprSpecYY_NUM_RULES 34
#define ExprSpecYY_END_OF_BUFFER 35
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct ExprSpec_trans_info
	{
	flex_int32_t ExprSpec_verify;
	flex_int32_t ExprSpec_nxt;
	};
static const flex_int16_t ExprSpec_accept[115] =
    {   0,
        0,    0,   35,   33,   31,   31,   33,   33,   32,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   23,   33,
       33,   33,   28,   28,   18,   28,   33,   33,   28,   28,
       28,   28,   28,   28,   33,    7,    0,   24,    0,   32,
        0,   26,   15,    5,    0,   25,    0,   13,   11,   12,
       10,   23,   14,   23,   23,    0,    8,    6,    9,   28,
       28,   17,   29,   30,   16,   28,   28,   28,    2,   28,
       28,    4,   24,   26,    0,   25,    0,   23,    0,    0,
       23,   28,   22,   28,   28,   28,   28,    0,    0,   23,
        0,    0,   23,   28,    3,   28,   28,   28,   27,    0,

       23,   28,   28,   28,   28,   27,   28,   28,   19,   20,
        1,   28,   21,    0
    } ;

static const ExprSpecYY_CHAR ExprSpec_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    5,    6,    7,    8,    9,   10,    1,
        1,   11,   12,    1,   13,   14,   15,   16,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   18,   19,   20,
       21,   22,    1,    1,   23,   24,   23,   23,   25,   23,
       26,   23,   27,   23,   23,   23,   23,   28,   23,   29,
       23,   23,   23,   23,   23,   23,   23,   23,   23,   23,
        1,   30,    1,   31,   23,    1,   32,   33,   23,   23,

       34,   35,   36,   37,   38,   23,   23,   39,   40,   41,
       42,   23,   23,   43,   44,   45,   46,   23,   23,   47,
       23,   23,    1,   48,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const ExprSpecYY_CHAR ExprSpec_meta[49] =
    {   0,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    3,    1,    3,    3,    4,    1,    1,
        1,    1,    5,    5,    5,    5,    5,    5,    5,    1,
        1,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    1
    } ;

static const flex_int16_t ExprSpec_base[123] =
    {   0,
        0,    0,  193,  194,  194,  194,  171,   44,  161,    0,
      169,  180,   40,  167,  166,   30,   37,  165,   41,  164,
      163,  162,    0,  157,    0,  154,   15,  159,  137,  139,
      145,  141,  137,  134,  125,  194,   54,  194,   56,  142,
      130,  152,  194,  194,   52,  194,   53,  194,  194,  194,
      194,   51,  194,   55,   74,   80,  194,  194,  194,    0,
      143,    0,  194,  194,  194,  121,  123,  120,    0,  124,
      122,  194,   64,  145,  144,   68,   88,   86,   97,   48,
       90,  134,    0,  126,  115,  124,  115,    0,   99,  101,
      109,  107,  111,  127,    0,  105,  112,   94,    0,  113,

      115,  118,   81,   69,   58,    0,   68,   49,    0,    0,
      194,   38,    0,  194,  132,  137,   72,  142,  145,  148,
       68,  151
    } ;

static const flex_int16_t ExprSpec_def[123] =
    {   0,
      114,    1,  114,  114,  114,  114,  114,  115,  116,  117,
      114,  114,  118,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  119,  119,  119,  119,  114,  114,  119,  119,
      119,  119,  119,  119,  114,  114,  115,  114,  115,  116,
      116,  120,  114,  114,  118,  114,  118,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  119,
      119,  119,  114,  114,  114,  119,  119,  119,  119,  119,
      119,  114,  115,  120,  114,  118,  114,  114,  114,  114,
      114,  119,  119,  119,  119,  119,  119,  121,  114,  114,
      114,  114,  114,  119,  119,  119,  119,  119,  122,  114,

      114,  119,  119,  119,  119,  122,  119,  119,  119,  119,
      114,  119,  119,    0,  114,  114,  114,  114,  114,  114,
      114,  114
    } ;

static const flex_int16_t ExprSpec_nxt[243] =
    {   0,
        4,    5,    6,    7,    8,    9,   10,   11,   12,   13,
       14,   15,   16,   17,   18,   19,   19,    4,    4,   20,
       21,   22,   23,   24,   25,   23,   23,   23,   26,   27,
       28,   23,   29,   30,   23,   31,   23,   32,   33,   23,
       23,   23,   23,   34,   23,   23,   23,   35,   38,   46,
       50,   51,   52,   52,   54,   63,   55,   55,   38,   64,
       73,   46,   76,   81,   81,   56,   52,   52,   38,   47,
       78,   78,   99,   39,   56,   77,   42,   46,  113,   79,
      112,   47,   47,   39,   77,   39,  111,   54,   79,   55,
       55,   80,   80,   39,  110,   81,   81,   47,   56,   89,

       89,   78,   78,   90,   90,   81,   81,   56,   92,   92,
       91,  109,   93,   93,   90,   90,   90,   90,  108,   91,
      100,  100,   93,   93,  101,  101,   93,   93,  101,  101,
      101,  101,   37,  107,   37,   37,   37,   40,  105,   40,
       40,   40,   45,  104,   45,   45,   45,   60,  103,   60,
       74,   74,   74,  106,  102,  106,   98,   97,   96,   95,
       94,   88,   75,   87,   86,   85,   84,   83,   82,   75,
      114,   41,   72,   71,   70,   69,   68,   67,   66,   65,
       62,   61,   59,   58,   57,   53,   49,   48,   44,   43,
       41,   36,  114,    3,  114,  114,  114,  114,  114,  114,

      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114
    } ;

static const flex_int16_t ExprSpec_chk[243] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    8,   13,
       16,   16,   17,   17,   19,   27,   19,   19,   37,   27,
       39,   45,   47,   80,   80,   19,   52,   52,   73,   13,
       54,   54,  121,    8,   19,   52,  117,   76,  112,   54,
      108,   45,   47,   37,   52,   39,  107,   55,   54,   55,
       55,   56,   56,   73,  105,   56,   56,   76,   55,   77,

       77,   78,   78,   77,   77,   81,   81,   55,   79,   79,
       78,  104,   79,   79,   89,   89,   90,   90,  103,   78,
       91,   91,   92,   92,   91,   91,   93,   93,  100,  100,
      101,  101,  115,  102,  115,  115,  115,  116,   98,  116,
      116,  116,  118,   97,  118,  118,  118,  119,   96,  119,
      120,  120,  120,  122,   94,  122,   87,   86,   85,   84,
       82,   75,   74,   71,   70,   68,   67,   66,   61,   42,
       41,   40,   35,   34,   33,   32,   31,   30,   29,   28,
       26,   24,   22,   21,   20,   18,   15,   14,   12,   11,
        9,    7,    3,  114,  114,  114,  114,  114,  114,  114,

      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114,  114,  114,  114,  114,  114,  114,  114,  114,
      114,  114
    } ;

static ExprSpec_state_type ExprSpec_last_accepting_state;
static char *ExprSpec_last_accepting_cpos;

extern int ExprSpec_flex_debug;
int ExprSpec_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define ExprSpecmore() ExprSpecmore_used_but_not_detected
#define ExprSpecYY_MORE_ADJ 0
#define ExprSpecYY_RESTORE_ExprSpecYY_MORE_OFFSET
char *ExprSpectext;
#line 1 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
/**
 * SPDX-FileCopyrightText: 2011-2019 Disney Enterprises, Inc.
 * SPDX-License-Identifier: LicenseRef-Apache-2.0
 * SPDX-FileCopyrightText: 2020-2021 L. E. Segovia <amy@amyspark.me>
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
/* Don't generate ExprSpecwrap since everything is in one string */
/* Don't generate unput since it's unused and gcc complains... */
/* Don't generate input since it's unused too -- amyspark */
#define ExprSpecYY_NO_INPUT 1
/* Don't worry about interactive and using isatty(). Fixes Windows compile. */
#line 18 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
#include <cctype>
#include <cstring>
#include <cmath>
#include <vector>

// If c++11 don't use register designator, lex and yacc need to go...
#if __cplusplus > 199711L
#define register      // Deprecated in C++11.
#endif  // #if __cplusplus > 199711L

#include "ExprSpecType.h"
#include <KSeExpr/Utils.h>

#ifdef SEEXPR_WIN32
#    define ExprSpecYY_NO_UNISTD_H
#    define ExprSpecYY_SKIP_ExprSpecYYWRAP
#endif

#ifndef MAKEDEPEND
#    include "ExprSpecParser.tab.h"
#endif

// TODO: make this thread safe
static int columnNumber=0; // really buffer position
static int lineNumber=0; // not used
static std::vector<std::pair<int,int> >* comments=0;

extern char* specRegisterToken(char* tok);
namespace KSeExpr {
void specResetCounters(std::vector<std::pair<int,int> >& commentsIn){
    columnNumber=lineNumber=0;
    comments=&commentsIn;
}
}

int ExprSpecpos(); // forward declare


#define ExprSpecYY_USER_ACTION { \
    ExprSpeclloc.first_line=lineNumber;ExprSpeclloc.first_column=columnNumber; \
    columnNumber+=ExprSpecleng;\
    ExprSpeclloc.last_column=columnNumber;ExprSpeclloc.last_line=lineNumber;} 

#line 593 "ExprSpecParserLexIn.cpp"
#line 594 "ExprSpecParserLexIn.cpp"

#define INITIAL 0

#ifndef ExprSpecYY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef ExprSpecYY_EXTRA_TYPE
#define ExprSpecYY_EXTRA_TYPE void *
#endif

static int ExprSpec_init_globals ( void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int ExprSpeclex_destroy ( void );

int ExprSpecget_debug ( void );

void ExprSpecset_debug ( int debug_flag  );

ExprSpecYY_EXTRA_TYPE ExprSpecget_extra ( void );

void ExprSpecset_extra ( ExprSpecYY_EXTRA_TYPE user_defined  );

FILE *ExprSpecget_in ( void );

void ExprSpecset_in  ( FILE * _in_str  );

FILE *ExprSpecget_out ( void );

void ExprSpecset_out  ( FILE * _out_str  );

			int ExprSpecget_leng ( void );

char *ExprSpecget_text ( void );

int ExprSpecget_lineno ( void );

void ExprSpecset_lineno ( int _line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef ExprSpecYY_SKIP_ExprSpecYYWRAP
#ifdef __cplusplus
extern "C" int ExprSpecwrap ( void );
#else
extern int ExprSpecwrap ( void );
#endif
#endif

#ifndef ExprSpecYY_NO_UNPUT
    
#endif

#ifndef ExprSpectext_ptr
static void ExprSpec_flex_strncpy ( char *, const char *, int );
#endif

#ifdef ExprSpecYY_NEED_STRLEN
static int ExprSpec_flex_strlen ( const char * );
#endif

#ifndef ExprSpecYY_NO_INPUT
#ifdef __cplusplus
static int ExprSpecinput ( void );
#else
static int input ( void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef ExprSpecYY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define ExprSpecYY_READ_BUF_SIZE 16384
#else
#define ExprSpecYY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( ExprSpectext, (size_t) ExprSpecleng, 1, ExprSpecout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or ExprSpecYY_NULL,
 * is returned in "result".
 */
#ifndef ExprSpecYY_INPUT
#define ExprSpecYY_INPUT(buf,result,max_size) \
	if ( ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( ExprSpecin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( ExprSpecin ) ) \
			ExprSpecYY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (ExprSpec_size_t) max_size, ExprSpecin)) == 0 && ferror(ExprSpecin)) \
			{ \
			if( errno != EINTR) \
				{ \
				ExprSpecYY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(ExprSpecin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "ExprSpecterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef ExprSpecterminate
#define ExprSpecterminate() return ExprSpecYY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef ExprSpecYY_START_STACK_INCR
#define ExprSpecYY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef ExprSpecYY_FATAL_ERROR
#define ExprSpecYY_FATAL_ERROR(msg) ExprSpec_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef ExprSpecYY_DECL
#define ExprSpecYY_DECL_IS_OURS 1

extern int ExprSpeclex (void);

#define ExprSpecYY_DECL int ExprSpeclex (void)
#endif /* !ExprSpecYY_DECL */

/* Code executed at the beginning of each rule, after ExprSpectext and ExprSpecleng
 * have been set up.
 */
#ifndef ExprSpecYY_USER_ACTION
#define ExprSpecYY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef ExprSpecYY_BREAK
#define ExprSpecYY_BREAK /*LINTED*/break;
#endif

#define ExprSpecYY_RULE_SETUP \
	ExprSpecYY_USER_ACTION

/** The main scanner function which does all the work.
 */
ExprSpecYY_DECL
{
	ExprSpec_state_type ExprSpec_current_state;
	char *ExprSpec_cp, *ExprSpec_bp;
	int ExprSpec_act;
    
	if ( !(ExprSpec_init) )
		{
		(ExprSpec_init) = 1;

#ifdef ExprSpecYY_USER_INIT
		ExprSpecYY_USER_INIT;
#endif

		if ( ! (ExprSpec_start) )
			(ExprSpec_start) = 1;	/* first start state */

		if ( ! ExprSpecin )
			ExprSpecin = stdin;

		if ( ! ExprSpecout )
			ExprSpecout = stdout;

		if ( ! ExprSpecYY_CURRENT_BUFFER ) {
			ExprSpecensure_buffer_stack ();
			ExprSpecYY_CURRENT_BUFFER_LVALUE =
				ExprSpec_create_buffer( ExprSpecin, ExprSpecYY_BUF_SIZE );
		}

		ExprSpec_load_buffer_state(  );
		}

	{
#line 68 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"

#line 811 "ExprSpecParserLexIn.cpp"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
		ExprSpec_cp = (ExprSpec_c_buf_p);

		/* Support of ExprSpectext. */
		*ExprSpec_cp = (ExprSpec_hold_char);

		/* ExprSpec_bp points to the position in ExprSpec_ch_buf of the start of
		 * the current run.
		 */
		ExprSpec_bp = ExprSpec_cp;

		ExprSpec_current_state = (ExprSpec_start);
ExprSpec_match:
		do
			{
			ExprSpecYY_CHAR ExprSpec_c = ExprSpec_ec[ExprSpecYY_SC_TO_UI(*ExprSpec_cp)] ;
			if ( ExprSpec_accept[ExprSpec_current_state] )
				{
				(ExprSpec_last_accepting_state) = ExprSpec_current_state;
				(ExprSpec_last_accepting_cpos) = ExprSpec_cp;
				}
			while ( ExprSpec_chk[ExprSpec_base[ExprSpec_current_state] + ExprSpec_c] != ExprSpec_current_state )
				{
				ExprSpec_current_state = (int) ExprSpec_def[ExprSpec_current_state];
				if ( ExprSpec_current_state >= 115 )
					ExprSpec_c = ExprSpec_meta[ExprSpec_c];
				}
			ExprSpec_current_state = ExprSpec_nxt[ExprSpec_base[ExprSpec_current_state] + ExprSpec_c];
			++ExprSpec_cp;
			}
		while ( ExprSpec_current_state != 114 );
		ExprSpec_cp = (ExprSpec_last_accepting_cpos);
		ExprSpec_current_state = (ExprSpec_last_accepting_state);

ExprSpec_find_action:
		ExprSpec_act = ExprSpec_accept[ExprSpec_current_state];

		ExprSpecYY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( ExprSpec_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of ExprSpecYY_DO_BEFORE_ACTION */
			*ExprSpec_cp = (ExprSpec_hold_char);
			ExprSpec_cp = (ExprSpec_last_accepting_cpos);
			ExprSpec_current_state = (ExprSpec_last_accepting_state);
			goto ExprSpec_find_action;

case 1:
ExprSpecYY_RULE_SETUP
#line 69 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"

	ExprSpecYY_BREAK
case 2:
ExprSpecYY_RULE_SETUP
#line 71 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return IF; }
	ExprSpecYY_BREAK
case 3:
ExprSpecYY_RULE_SETUP
#line 72 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return ELSE; }
	ExprSpecYY_BREAK
case 4:
ExprSpecYY_RULE_SETUP
#line 74 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return OR; }
	ExprSpecYY_BREAK
case 5:
ExprSpecYY_RULE_SETUP
#line 75 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return AND; }
	ExprSpecYY_BREAK
case 6:
ExprSpecYY_RULE_SETUP
#line 76 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return EQ; }
	ExprSpecYY_BREAK
case 7:
ExprSpecYY_RULE_SETUP
#line 77 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return NE; }
	ExprSpecYY_BREAK
case 8:
ExprSpecYY_RULE_SETUP
#line 78 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return LE; }
	ExprSpecYY_BREAK
case 9:
ExprSpecYY_RULE_SETUP
#line 79 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return GE; }
	ExprSpecYY_BREAK
case 10:
ExprSpecYY_RULE_SETUP
#line 80 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return ARROW; }
	ExprSpecYY_BREAK
case 11:
ExprSpecYY_RULE_SETUP
#line 81 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return AddEq; }
	ExprSpecYY_BREAK
case 12:
ExprSpecYY_RULE_SETUP
#line 82 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return SubEq; }
	ExprSpecYY_BREAK
case 13:
ExprSpecYY_RULE_SETUP
#line 83 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return MultEq; }
	ExprSpecYY_BREAK
case 14:
ExprSpecYY_RULE_SETUP
#line 84 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return DivEq; }
	ExprSpecYY_BREAK
case 15:
ExprSpecYY_RULE_SETUP
#line 85 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return ModEq; }
	ExprSpecYY_BREAK
case 16:
ExprSpecYY_RULE_SETUP
#line 86 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return ExpEq; }
	ExprSpecYY_BREAK
case 17:
ExprSpecYY_RULE_SETUP
#line 88 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = M_PI; return NUMBER; }
	ExprSpecYY_BREAK
case 18:
ExprSpecYY_RULE_SETUP
#line 89 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = M_E; return NUMBER; }
	ExprSpecYY_BREAK
case 19:
ExprSpecYY_RULE_SETUP
#line 90 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = 0; return NUMBER; }
	ExprSpecYY_BREAK
case 20:
ExprSpecYY_RULE_SETUP
#line 91 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = 1; return NUMBER; }
	ExprSpecYY_BREAK
case 21:
ExprSpecYY_RULE_SETUP
#line 92 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = 2; return NUMBER; }
	ExprSpecYY_BREAK
case 22:
ExprSpecYY_RULE_SETUP
#line 93 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = 3; return NUMBER; }
	ExprSpecYY_BREAK
case 23:
ExprSpecYY_RULE_SETUP
#line 95 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.d = KSeExpr::Utils::atof(ExprSpectext); return NUMBER; }
	ExprSpecYY_BREAK
case 24:
ExprSpecYY_RULE_SETUP
#line 96 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ /* match quoted string, allow embedded quote, \" */
			  ExprSpeclval.s = specRegisterToken(&ExprSpectext[1]); 
			  ExprSpeclval.s[strlen(ExprSpeclval.s)-1] = '\0';
                          return STR; }
	ExprSpecYY_BREAK
case 25:
ExprSpecYY_RULE_SETUP
#line 100 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ /* match quoted string, allow embedded quote, \' */
			  ExprSpeclval.s = specRegisterToken(&ExprSpectext[1]); 
			  ExprSpeclval.s[strlen(ExprSpeclval.s)-1] = '\0';
                          return STR; }
	ExprSpecYY_BREAK
case 26:
ExprSpecYY_RULE_SETUP
#line 104 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.s = specRegisterToken(&ExprSpectext[1]); return VAR; }
	ExprSpecYY_BREAK
case 27:
ExprSpecYY_RULE_SETUP
#line 105 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.s = specRegisterToken(&ExprSpectext[1]); return VAR; }
	ExprSpecYY_BREAK
case 28:
ExprSpecYY_RULE_SETUP
#line 106 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ ExprSpeclval.s = specRegisterToken(ExprSpectext); return NAME; }
	ExprSpecYY_BREAK
case 29:
ExprSpecYY_RULE_SETUP
#line 108 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
/* ignore quoted newline */;
	ExprSpecYY_BREAK
case 30:
ExprSpecYY_RULE_SETUP
#line 109 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
/* ignore quoted tab */;
	ExprSpecYY_BREAK
case 31:
/* rule 31 can match eol */
ExprSpecYY_RULE_SETUP
#line 110 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
/* ignore whitespace */;
	ExprSpecYY_BREAK
case 32:
ExprSpecYY_RULE_SETUP
#line 111 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ /* match comment */ 
                         int startPos=ExprSpecpos(),endPos=ExprSpecpos()+strlen(&ExprSpectext[1])+1;
                          comments->push_back(std::pair<int,int>(startPos,endPos));}
	ExprSpecYY_BREAK
case 33:
ExprSpecYY_RULE_SETUP
#line 116 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
{ return ExprSpectext[0]; }
	ExprSpecYY_BREAK
case 34:
ExprSpecYY_RULE_SETUP
#line 118 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"
ECHO;
	ExprSpecYY_BREAK
#line 1043 "ExprSpecParserLexIn.cpp"
case ExprSpecYY_STATE_EOF(INITIAL):
	ExprSpecterminate();

	case ExprSpecYY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int ExprSpec_amount_of_matched_text = (int) (ExprSpec_cp - (ExprSpectext_ptr)) - 1;

		/* Undo the effects of ExprSpecYY_DO_BEFORE_ACTION. */
		*ExprSpec_cp = (ExprSpec_hold_char);
		ExprSpecYY_RESTORE_ExprSpecYY_MORE_OFFSET

		if ( ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buffer_status == ExprSpecYY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed ExprSpecin at a new source and called
			 * ExprSpeclex().  If so, then we have to assure
			 * consistency between ExprSpecYY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(ExprSpec_n_chars) = ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_n_chars;
			ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_input_file = ExprSpecin;
			ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buffer_status = ExprSpecYY_BUFFER_NORMAL;
			}

		/* Note that here we test for ExprSpec_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since ExprSpec_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (ExprSpec_c_buf_p) <= &ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[(ExprSpec_n_chars)] )
			{ /* This was really a NUL. */
			ExprSpec_state_type ExprSpec_next_state;

			(ExprSpec_c_buf_p) = (ExprSpectext_ptr) + ExprSpec_amount_of_matched_text;

			ExprSpec_current_state = ExprSpec_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * ExprSpec_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			ExprSpec_next_state = ExprSpec_try_NUL_trans( ExprSpec_current_state );

			ExprSpec_bp = (ExprSpectext_ptr) + ExprSpecYY_MORE_ADJ;

			if ( ExprSpec_next_state )
				{
				/* Consume the NUL. */
				ExprSpec_cp = ++(ExprSpec_c_buf_p);
				ExprSpec_current_state = ExprSpec_next_state;
				goto ExprSpec_match;
				}

			else
				{
				ExprSpec_cp = (ExprSpec_last_accepting_cpos);
				ExprSpec_current_state = (ExprSpec_last_accepting_state);
				goto ExprSpec_find_action;
				}
			}

		else switch ( ExprSpec_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(ExprSpec_did_buffer_switch_on_eof) = 0;

				if ( ExprSpecwrap(  ) )
					{
					/* Note: because we've taken care in
					 * ExprSpec_get_next_buffer() to have set up
					 * ExprSpectext, we can now set up
					 * ExprSpec_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * ExprSpecYY_NULL, it'll still work - another
					 * ExprSpecYY_NULL will get returned.
					 */
					(ExprSpec_c_buf_p) = (ExprSpectext_ptr) + ExprSpecYY_MORE_ADJ;

					ExprSpec_act = ExprSpecYY_STATE_EOF(ExprSpecYY_START);
					goto do_action;
					}

				else
					{
					if ( ! (ExprSpec_did_buffer_switch_on_eof) )
						ExprSpecYY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(ExprSpec_c_buf_p) =
					(ExprSpectext_ptr) + ExprSpec_amount_of_matched_text;

				ExprSpec_current_state = ExprSpec_get_previous_state(  );

				ExprSpec_cp = (ExprSpec_c_buf_p);
				ExprSpec_bp = (ExprSpectext_ptr) + ExprSpecYY_MORE_ADJ;
				goto ExprSpec_match;

			case EOB_ACT_LAST_MATCH:
				(ExprSpec_c_buf_p) =
				&ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[(ExprSpec_n_chars)];

				ExprSpec_current_state = ExprSpec_get_previous_state(  );

				ExprSpec_cp = (ExprSpec_c_buf_p);
				ExprSpec_bp = (ExprSpectext_ptr) + ExprSpecYY_MORE_ADJ;
				goto ExprSpec_find_action;
			}
		break;
		}

	default:
		ExprSpecYY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of ExprSpeclex */

/* ExprSpec_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int ExprSpec_get_next_buffer (void)
{
    	char *dest = ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf;
	char *source = (ExprSpectext_ptr);
	int number_to_move, i;
	int ret_val;

	if ( (ExprSpec_c_buf_p) > &ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[(ExprSpec_n_chars) + 1] )
		ExprSpecYY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (ExprSpec_c_buf_p) - (ExprSpectext_ptr) - ExprSpecYY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((ExprSpec_c_buf_p) - (ExprSpectext_ptr) - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buffer_status == ExprSpecYY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_n_chars = (ExprSpec_n_chars) = 0;

	else
		{
			int num_to_read =
			ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			ExprSpecYY_BUFFER_STATE b = ExprSpecYY_CURRENT_BUFFER_LVALUE;

			int ExprSpec_c_buf_p_offset =
				(int) ((ExprSpec_c_buf_p) - b->ExprSpec_ch_buf);

			if ( b->ExprSpec_is_our_buffer )
				{
				int new_size = b->ExprSpec_buf_size * 2;

				if ( new_size <= 0 )
					b->ExprSpec_buf_size += b->ExprSpec_buf_size / 8;
				else
					b->ExprSpec_buf_size *= 2;

				b->ExprSpec_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					ExprSpecrealloc( (void *) b->ExprSpec_ch_buf,
							 (ExprSpec_size_t) (b->ExprSpec_buf_size + 2)  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->ExprSpec_ch_buf = NULL;

			if ( ! b->ExprSpec_ch_buf )
				ExprSpecYY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(ExprSpec_c_buf_p) = &b->ExprSpec_ch_buf[ExprSpec_c_buf_p_offset];

			num_to_read = ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > ExprSpecYY_READ_BUF_SIZE )
			num_to_read = ExprSpecYY_READ_BUF_SIZE;

		/* Read in more data. */
		ExprSpecYY_INPUT( (&ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[number_to_move]),
			(ExprSpec_n_chars), num_to_read );

		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_n_chars = (ExprSpec_n_chars);
		}

	if ( (ExprSpec_n_chars) == 0 )
		{
		if ( number_to_move == ExprSpecYY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			ExprSpecrestart( ExprSpecin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buffer_status =
				ExprSpecYY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if (((ExprSpec_n_chars) + number_to_move) > ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = (ExprSpec_n_chars) + number_to_move + ((ExprSpec_n_chars) >> 1);
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf = (char *) ExprSpecrealloc(
			(void *) ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf, (ExprSpec_size_t) new_size  );
		if ( ! ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf )
			ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpec_get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_size = (int) (new_size - 2);
	}

	(ExprSpec_n_chars) += number_to_move;
	ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[(ExprSpec_n_chars)] = ExprSpecYY_END_OF_BUFFER_CHAR;
	ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[(ExprSpec_n_chars) + 1] = ExprSpecYY_END_OF_BUFFER_CHAR;

	(ExprSpectext_ptr) = &ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[0];

	return ret_val;
}

/* ExprSpec_get_previous_state - get the state just before the EOB char was reached */

    static ExprSpec_state_type ExprSpec_get_previous_state (void)
{
	ExprSpec_state_type ExprSpec_current_state;
	char *ExprSpec_cp;
    
	ExprSpec_current_state = (ExprSpec_start);

	for ( ExprSpec_cp = (ExprSpectext_ptr) + ExprSpecYY_MORE_ADJ; ExprSpec_cp < (ExprSpec_c_buf_p); ++ExprSpec_cp )
		{
		ExprSpecYY_CHAR ExprSpec_c = (*ExprSpec_cp ? ExprSpec_ec[ExprSpecYY_SC_TO_UI(*ExprSpec_cp)] : 1);
		if ( ExprSpec_accept[ExprSpec_current_state] )
			{
			(ExprSpec_last_accepting_state) = ExprSpec_current_state;
			(ExprSpec_last_accepting_cpos) = ExprSpec_cp;
			}
		while ( ExprSpec_chk[ExprSpec_base[ExprSpec_current_state] + ExprSpec_c] != ExprSpec_current_state )
			{
			ExprSpec_current_state = (int) ExprSpec_def[ExprSpec_current_state];
			if ( ExprSpec_current_state >= 115 )
				ExprSpec_c = ExprSpec_meta[ExprSpec_c];
			}
		ExprSpec_current_state = ExprSpec_nxt[ExprSpec_base[ExprSpec_current_state] + ExprSpec_c];
		}

	return ExprSpec_current_state;
}

/* ExprSpec_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = ExprSpec_try_NUL_trans( current_state );
 */
    static ExprSpec_state_type ExprSpec_try_NUL_trans  (ExprSpec_state_type ExprSpec_current_state )
{
	int ExprSpec_is_jam;
    	char *ExprSpec_cp = (ExprSpec_c_buf_p);

	ExprSpecYY_CHAR ExprSpec_c = 1;
	if ( ExprSpec_accept[ExprSpec_current_state] )
		{
		(ExprSpec_last_accepting_state) = ExprSpec_current_state;
		(ExprSpec_last_accepting_cpos) = ExprSpec_cp;
		}
	while ( ExprSpec_chk[ExprSpec_base[ExprSpec_current_state] + ExprSpec_c] != ExprSpec_current_state )
		{
		ExprSpec_current_state = (int) ExprSpec_def[ExprSpec_current_state];
		if ( ExprSpec_current_state >= 115 )
			ExprSpec_c = ExprSpec_meta[ExprSpec_c];
		}
	ExprSpec_current_state = ExprSpec_nxt[ExprSpec_base[ExprSpec_current_state] + ExprSpec_c];
	ExprSpec_is_jam = (ExprSpec_current_state == 114);

		return ExprSpec_is_jam ? 0 : ExprSpec_current_state;
}

#ifndef ExprSpecYY_NO_UNPUT

#endif

#ifndef ExprSpecYY_NO_INPUT
#ifdef __cplusplus
    static int ExprSpecinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(ExprSpec_c_buf_p) = (ExprSpec_hold_char);

	if ( *(ExprSpec_c_buf_p) == ExprSpecYY_END_OF_BUFFER_CHAR )
		{
		/* ExprSpec_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (ExprSpec_c_buf_p) < &ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_ch_buf[(ExprSpec_n_chars)] )
			/* This was really a NUL. */
			*(ExprSpec_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (int) ((ExprSpec_c_buf_p) - (ExprSpectext_ptr));
			++(ExprSpec_c_buf_p);

			switch ( ExprSpec_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because ExprSpec_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					ExprSpecrestart( ExprSpecin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( ExprSpecwrap(  ) )
						return 0;

					if ( ! (ExprSpec_did_buffer_switch_on_eof) )
						ExprSpecYY_NEW_FILE;
#ifdef __cplusplus
					return ExprSpecinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(ExprSpec_c_buf_p) = (ExprSpectext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (ExprSpec_c_buf_p);	/* cast for 8-bit char's */
	*(ExprSpec_c_buf_p) = '\0';	/* preserve ExprSpectext */
	(ExprSpec_hold_char) = *++(ExprSpec_c_buf_p);

	return c;
}
#endif	/* ifndef ExprSpecYY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void ExprSpecrestart  (FILE * input_file )
{
    
	if ( ! ExprSpecYY_CURRENT_BUFFER ){
        ExprSpecensure_buffer_stack ();
		ExprSpecYY_CURRENT_BUFFER_LVALUE =
            ExprSpec_create_buffer( ExprSpecin, ExprSpecYY_BUF_SIZE );
	}

	ExprSpec_init_buffer( ExprSpecYY_CURRENT_BUFFER, input_file );
	ExprSpec_load_buffer_state(  );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void ExprSpec_switch_to_buffer  (ExprSpecYY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		ExprSpecpop_buffer_state();
	 *		ExprSpecpush_buffer_state(new_buffer);
     */
	ExprSpecensure_buffer_stack ();
	if ( ExprSpecYY_CURRENT_BUFFER == new_buffer )
		return;

	if ( ExprSpecYY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(ExprSpec_c_buf_p) = (ExprSpec_hold_char);
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_pos = (ExprSpec_c_buf_p);
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_n_chars = (ExprSpec_n_chars);
		}

	ExprSpecYY_CURRENT_BUFFER_LVALUE = new_buffer;
	ExprSpec_load_buffer_state(  );

	/* We don't actually know whether we did this switch during
	 * EOF (ExprSpecwrap()) processing, but the only time this flag
	 * is looked at is after ExprSpecwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(ExprSpec_did_buffer_switch_on_eof) = 1;
}

static void ExprSpec_load_buffer_state  (void)
{
    	(ExprSpec_n_chars) = ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_n_chars;
	(ExprSpectext_ptr) = (ExprSpec_c_buf_p) = ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_pos;
	ExprSpecin = ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_input_file;
	(ExprSpec_hold_char) = *(ExprSpec_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c ExprSpecYY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    ExprSpecYY_BUFFER_STATE ExprSpec_create_buffer  (FILE * file, int  size )
{
	ExprSpecYY_BUFFER_STATE b;
    
	b = (ExprSpecYY_BUFFER_STATE) ExprSpecalloc( sizeof( struct ExprSpec_buffer_state )  );
	if ( ! b )
		ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpec_create_buffer()" );

	b->ExprSpec_buf_size = size;

	/* ExprSpec_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->ExprSpec_ch_buf = (char *) ExprSpecalloc( (ExprSpec_size_t) (b->ExprSpec_buf_size + 2)  );
	if ( ! b->ExprSpec_ch_buf )
		ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpec_create_buffer()" );

	b->ExprSpec_is_our_buffer = 1;

	ExprSpec_init_buffer( b, file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with ExprSpec_create_buffer()
 * 
 */
    void ExprSpec_delete_buffer (ExprSpecYY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == ExprSpecYY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		ExprSpecYY_CURRENT_BUFFER_LVALUE = (ExprSpecYY_BUFFER_STATE) 0;

	if ( b->ExprSpec_is_our_buffer )
		ExprSpecfree( (void *) b->ExprSpec_ch_buf  );

	ExprSpecfree( (void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a ExprSpecrestart() or at EOF.
 */
    static void ExprSpec_init_buffer  (ExprSpecYY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	ExprSpec_flush_buffer( b );

	b->ExprSpec_input_file = file;
	b->ExprSpec_fill_buffer = 1;

    /* If b is the current buffer, then ExprSpec_init_buffer was _probably_
     * called from ExprSpecrestart() or through ExprSpec_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != ExprSpecYY_CURRENT_BUFFER){
        b->ExprSpec_bs_lineno = 1;
        b->ExprSpec_bs_column = 0;
    }

        b->ExprSpec_is_interactive = 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, ExprSpecYY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c ExprSpecYY_CURRENT_BUFFER.
 * 
 */
    void ExprSpec_flush_buffer (ExprSpecYY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->ExprSpec_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->ExprSpec_ch_buf[0] = ExprSpecYY_END_OF_BUFFER_CHAR;
	b->ExprSpec_ch_buf[1] = ExprSpecYY_END_OF_BUFFER_CHAR;

	b->ExprSpec_buf_pos = &b->ExprSpec_ch_buf[0];

	b->ExprSpec_at_bol = 1;
	b->ExprSpec_buffer_status = ExprSpecYY_BUFFER_NEW;

	if ( b == ExprSpecYY_CURRENT_BUFFER )
		ExprSpec_load_buffer_state(  );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void ExprSpecpush_buffer_state (ExprSpecYY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	ExprSpecensure_buffer_stack();

	/* This block is copied from ExprSpec_switch_to_buffer. */
	if ( ExprSpecYY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(ExprSpec_c_buf_p) = (ExprSpec_hold_char);
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_buf_pos = (ExprSpec_c_buf_p);
		ExprSpecYY_CURRENT_BUFFER_LVALUE->ExprSpec_n_chars = (ExprSpec_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (ExprSpecYY_CURRENT_BUFFER)
		(ExprSpec_buffer_stack_top)++;
	ExprSpecYY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from ExprSpec_switch_to_buffer. */
	ExprSpec_load_buffer_state(  );
	(ExprSpec_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void ExprSpecpop_buffer_state (void)
{
    	if (!ExprSpecYY_CURRENT_BUFFER)
		return;

	ExprSpec_delete_buffer(ExprSpecYY_CURRENT_BUFFER );
	ExprSpecYY_CURRENT_BUFFER_LVALUE = NULL;
	if ((ExprSpec_buffer_stack_top) > 0)
		--(ExprSpec_buffer_stack_top);

	if (ExprSpecYY_CURRENT_BUFFER) {
		ExprSpec_load_buffer_state(  );
		(ExprSpec_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void ExprSpecensure_buffer_stack (void)
{
	ExprSpec_size_t num_to_alloc;
    
	if (!(ExprSpec_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		(ExprSpec_buffer_stack) = (struct ExprSpec_buffer_state**)ExprSpecalloc
								(num_to_alloc * sizeof(struct ExprSpec_buffer_state*)
								);
		if ( ! (ExprSpec_buffer_stack) )
			ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpecensure_buffer_stack()" );

		memset((ExprSpec_buffer_stack), 0, num_to_alloc * sizeof(struct ExprSpec_buffer_state*));

		(ExprSpec_buffer_stack_max) = num_to_alloc;
		(ExprSpec_buffer_stack_top) = 0;
		return;
	}

	if ((ExprSpec_buffer_stack_top) >= ((ExprSpec_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		ExprSpec_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (ExprSpec_buffer_stack_max) + grow_size;
		(ExprSpec_buffer_stack) = (struct ExprSpec_buffer_state**)ExprSpecrealloc
								((ExprSpec_buffer_stack),
								num_to_alloc * sizeof(struct ExprSpec_buffer_state*)
								);
		if ( ! (ExprSpec_buffer_stack) )
			ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpecensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((ExprSpec_buffer_stack) + (ExprSpec_buffer_stack_max), 0, grow_size * sizeof(struct ExprSpec_buffer_state*));
		(ExprSpec_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object.
 */
ExprSpecYY_BUFFER_STATE ExprSpec_scan_buffer  (char * base, ExprSpec_size_t  size )
{
	ExprSpecYY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != ExprSpecYY_END_OF_BUFFER_CHAR ||
	     base[size-1] != ExprSpecYY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (ExprSpecYY_BUFFER_STATE) ExprSpecalloc( sizeof( struct ExprSpec_buffer_state )  );
	if ( ! b )
		ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpec_scan_buffer()" );

	b->ExprSpec_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->ExprSpec_buf_pos = b->ExprSpec_ch_buf = base;
	b->ExprSpec_is_our_buffer = 0;
	b->ExprSpec_input_file = NULL;
	b->ExprSpec_n_chars = b->ExprSpec_buf_size;
	b->ExprSpec_is_interactive = 0;
	b->ExprSpec_at_bol = 1;
	b->ExprSpec_fill_buffer = 0;
	b->ExprSpec_buffer_status = ExprSpecYY_BUFFER_NEW;

	ExprSpec_switch_to_buffer( b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to ExprSpeclex() will
 * scan from a @e copy of @a str.
 * @param ExprSpecstr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       ExprSpec_scan_bytes() instead.
 */
ExprSpecYY_BUFFER_STATE ExprSpec_scan_string (const char * ExprSpecstr )
{
    
	return ExprSpec_scan_bytes( ExprSpecstr, (int) strlen(ExprSpecstr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to ExprSpeclex() will
 * scan from a @e copy of @a bytes.
 * @param ExprSpecbytes the byte buffer to scan
 * @param _ExprSpecbytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
ExprSpecYY_BUFFER_STATE ExprSpec_scan_bytes  (const char * ExprSpecbytes, int  _ExprSpecbytes_len )
{
	ExprSpecYY_BUFFER_STATE b;
	char *buf;
	ExprSpec_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (ExprSpec_size_t) (_ExprSpecbytes_len + 2);
	buf = (char *) ExprSpecalloc( n  );
	if ( ! buf )
		ExprSpecYY_FATAL_ERROR( "out of dynamic memory in ExprSpec_scan_bytes()" );

	for ( i = 0; i < _ExprSpecbytes_len; ++i )
		buf[i] = ExprSpecbytes[i];

	buf[_ExprSpecbytes_len] = buf[_ExprSpecbytes_len+1] = ExprSpecYY_END_OF_BUFFER_CHAR;

	b = ExprSpec_scan_buffer( buf, n );
	if ( ! b )
		ExprSpecYY_FATAL_ERROR( "bad buffer in ExprSpec_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->ExprSpec_is_our_buffer = 1;

	return b;
}

#ifndef ExprSpecYY_EXIT_FAILURE
#define ExprSpecYY_EXIT_FAILURE 2
#endif

static void ExprSpecnoreturn ExprSpec_fatal_error (const char* msg )
{
			fprintf( stderr, "%s\n", msg );
	exit( ExprSpecYY_EXIT_FAILURE );
}

/* Redefine ExprSpecless() so it works in section 3 code. */

#undef ExprSpecless
#define ExprSpecless(n) \
	do \
		{ \
		/* Undo effects of setting up ExprSpectext. */ \
        int ExprSpecless_macro_arg = (n); \
        ExprSpecYY_LESS_LINENO(ExprSpecless_macro_arg);\
		ExprSpectext[ExprSpecleng] = (ExprSpec_hold_char); \
		(ExprSpec_c_buf_p) = ExprSpectext + ExprSpecless_macro_arg; \
		(ExprSpec_hold_char) = *(ExprSpec_c_buf_p); \
		*(ExprSpec_c_buf_p) = '\0'; \
		ExprSpecleng = ExprSpecless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int ExprSpecget_lineno  (void)
{
    
    return ExprSpeclineno;
}

/** Get the input stream.
 * 
 */
FILE *ExprSpecget_in  (void)
{
        return ExprSpecin;
}

/** Get the output stream.
 * 
 */
FILE *ExprSpecget_out  (void)
{
        return ExprSpecout;
}

/** Get the length of the current token.
 * 
 */
int ExprSpecget_leng  (void)
{
        return ExprSpecleng;
}

/** Get the current token.
 * 
 */

char *ExprSpecget_text  (void)
{
        return ExprSpectext;
}

/** Set the current line number.
 * @param _line_number line number
 * 
 */
void ExprSpecset_lineno (int  _line_number )
{
    
    ExprSpeclineno = _line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * 
 * @see ExprSpec_switch_to_buffer
 */
void ExprSpecset_in (FILE *  _in_str )
{
        ExprSpecin = _in_str ;
}

void ExprSpecset_out (FILE *  _out_str )
{
        ExprSpecout = _out_str ;
}

int ExprSpecget_debug  (void)
{
        return ExprSpec_flex_debug;
}

void ExprSpecset_debug (int  _bdebug )
{
        ExprSpec_flex_debug = _bdebug ;
}

static int ExprSpec_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from ExprSpeclex_destroy(), so don't allocate here.
     */

    (ExprSpec_buffer_stack) = NULL;
    (ExprSpec_buffer_stack_top) = 0;
    (ExprSpec_buffer_stack_max) = 0;
    (ExprSpec_c_buf_p) = NULL;
    (ExprSpec_init) = 0;
    (ExprSpec_start) = 0;

/* Defined in main.c */
#ifdef ExprSpecYY_STDINIT
    ExprSpecin = stdin;
    ExprSpecout = stdout;
#else
    ExprSpecin = NULL;
    ExprSpecout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * ExprSpeclex_init()
     */
    return 0;
}

/* ExprSpeclex_destroy is for both reentrant and non-reentrant scanners. */
int ExprSpeclex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(ExprSpecYY_CURRENT_BUFFER){
		ExprSpec_delete_buffer( ExprSpecYY_CURRENT_BUFFER  );
		ExprSpecYY_CURRENT_BUFFER_LVALUE = NULL;
		ExprSpecpop_buffer_state();
	}

	/* Destroy the stack itself. */
	ExprSpecfree((ExprSpec_buffer_stack) );
	(ExprSpec_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * ExprSpeclex() is called, initialization will occur. */
    ExprSpec_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef ExprSpectext_ptr
static void ExprSpec_flex_strncpy (char* s1, const char * s2, int n )
{
		
	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef ExprSpecYY_NEED_STRLEN
static int ExprSpec_flex_strlen (const char * s )
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *ExprSpecalloc (ExprSpec_size_t  size )
{
			return malloc(size);
}

void *ExprSpecrealloc  (void * ptr, ExprSpec_size_t  size )
{
		
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void ExprSpecfree (void * ptr )
{
			free( (char *) ptr );	/* see ExprSpecrealloc() for (char *) cast */
}

#define ExprSpecYYTABLES_NAME "ExprSpectables"

#line 118 "@@PATH@@/src/KSeExprUI/ExprSpecParserLex.l"


/* Gets index of current token (corresponding to ExprSpectext).  
   Used for error reporting.
 */
int ExprSpecpos()
{
    return ExprSpec_c_buf_p - ExprSpecYY_CURRENT_BUFFER->ExprSpec_ch_buf - ExprSpecleng;
}

